@@ -1,6 +1,6 @@
 package PDF::Create::Page;
 
-our $VERSION = '1.19';
+our $VERSION = '1.20';
 
 =encoding utf8
 
@@ -10,7 +10,7 @@ PDF::Create::Page - PDF pages tree for PDF::Create
 
 =head1 VERSION
 
-Version 1.19
+Version 1.20
 
 =cut
 
@@ -370,6 +370,29 @@ sub setrgbcolorstroke {
 
 =head2 text(%params)
 
+Renders the text. Parameters are explained as below:
+
+    +--------+------------------------------------------------------------------+
+    | Key    | Description                                                      |
+    +--------+------------------------------------------------------------------+
+    | start  | The start marker, add directive BT                               |
+    | end    | The end marker, add directive ET                                 |
+    | text   | Text to add to the pdf                                           |
+    | F      | Font index to be used, add directive /F<font_index>              |
+    | Tf     | Font size for the text, add directive <font_size> Tf             |
+    | Ts     | Text rise (super/subscript), add directive <mode> Ts             |
+    | Tr     | Text rendering mode, add directive <mode> Tr                     |
+    | TL     | Text leading, add directive <number> TL                          |
+    | Tc     | Character spacing, add directive <number> Tc                     |
+    | Tw     | Word spacing, add directive <number> Tw                          |
+    | Tz     | Horizontal scaling, add directive <number> Tz                    |
+    | Td     | Move to, add directive <x> <y> Td                                |
+    | TD     | Move to and set TL, add directive <x> <y> TD                     |
+    | rot    | Move to and rotate (<r> <x> <y>), add directive                  |
+    |        | <cos(r)>, <sin(r)>, <sin(r)>, <cos(r)>, <x>, <y> Tm              |
+    | T*     | Add new line.                                                    |
+    +--------+------------------------------------------------------------------+
+
 =cut
 
 sub text {
@@ -377,20 +400,33 @@ sub text {
 
     PDF::Create::debug( 2, "text(%params):" );
 
-    if (defined $params{'start'}) { $ptext = "BT "; }
+    my @directives = ();
 
+    if (defined $params{'start'}) { push @directives, "BT"; }
+
+    # Font index
+    if (defined $params{'F'})  {
+        push @directives, "/F$params{'F'}";
+        $self->{'pdf'}->uses_font($self, $params{'F'});
+    }
+    # Font size
+    if (defined $params{'Tf'}) { push @directives, "$params{'Tf'} Tf"; }
     # Text Rise (Super/Subscript)
-    if (defined $params{'Ts'})    { $ptext .= " $params{'Ts'} Ts "; }
+    if (defined $params{'Ts'}) { push @directives, "$params{'Ts'} Ts"; }
     # Rendering Mode
-    if (defined $params{'Tr'})    { $ptext .= " $params{'Tr'} Tr "; }
+    if (defined $params{'Tr'}) { push @directives, "$params{'Tr'} Tr"; }
     # Text Leading
-    if (defined $params{'TL'})    { $ptext .= " $params{'TL'} TL "; }
+    if (defined $params{'TL'}) { push @directives, "$params{'TL'} TL"; }
     # Character spacing
-    if (defined $params{'Tc'})    { $ptext .= " $params{'Tc'} Tc "; }
+    if (defined $params{'Tc'}) { push @directives, "$params{'Tc'} Tc"; }
     # Word Spacing
-    if (defined $params{'Tw'})    { $ptext .= " $params{'Tw'} Tw "; }
+    if (defined $params{'Tw'}) { push @directives, "$params{'Tw'} Tw"; }
     # Horizontal Scaling
-    if (defined $params{'Tz'})    { $ptext .= " $params{'Tz'} Tz "; }
+    if (defined $params{'Tz'}) { push @directives, "$params{'Tz'} Tz"; }
+    # Moveto
+    if (defined $params{'Td'}) { push @directives, "$params{'Td'} Td"; }
+    # Moveto and set TL
+    if (defined $params{'TD'}) { push @directives, "$params{'TD'} TD"; }
 
     # Moveto and rotateOA
     my $pi = atan2(1, 1) * 4;
@@ -401,27 +437,22 @@ sub text {
         $y = 0 unless ($y > 0);
         my $cos = cos($r * $piover180);
         my $sin = sin($r * $piover180);
-        $ptext .= sprintf(" %.5f %.5f -%.5f %.5f %s %s Tm ", $cos, $sin, $sin, $cos, $x, $y);
+        push @directives, sprintf("%.5f %.5f -%.5f %.5f %s %s Tm", $cos, $sin, $sin, $cos, $x, $y);
     }
 
-    # Font size
-    if (defined $params{'Tf'}) { $ptext .= "/F$params{'Tf'} Tf "; }
-    # Moveto
-    if (defined $params{'Td'}) { $ptext .= " $params{'Td'} Td "; }
-    # Moveto and set TL
-    if (defined $params{'TD'}) { $ptext .= " $params{'TD'} TD "; }
     # New line
-    if (defined $params{'T*'}) { $ptext .= " T* "; }
+    if (defined $params{'T*'}) { push @directives, "T*"; }
 
     if (defined $params{'text'}) {
         $params{'text'} =~ s|([()])|\\$1|g;
-        $ptext .= "($params{'text'}) Tj ";
+        push @directives, "($params{'text'}) Tj";
     }
 
     if (defined $params{'end'}) {
-        $ptext .= " ET";
+        push @directives, "ET";
+        $ptext = join(' ', @directives);
         $self->{'pdf'}->page_stream($self);
-        $self->{'pdf'}->add("$ptext");
+        $self->{'pdf'}->add($ptext);
     }
 
     PDF::Create::debug( 3, "text(): $ptext" );
@@ -449,9 +480,10 @@ Example :
 =cut
 
 sub string {
-    my ($self, $font, $size, $x, $y, $string, $align) = @_;
+    my ($self, $font, $size, $x, $y, $string, $align,
+        $char_spacing, $word_spacing) = @_;
 
-    $align = 'L' unless defined $align;;
+    $align = 'L' unless defined $align;
 
     if (uc($align) eq "R") {
         $x -= $size * $self->string_width($font, $string);
@@ -459,10 +491,30 @@ sub string {
         $x -= $size * $self->string_width($font, $string) / 2;
     }
 
+    my @directives = (
+        'BT',
+        "/F$font",
+        "$size Tf",
+    );
+
+    if (defined $char_spacing && $char_spacing =~ m/[0-9]+\.?[0-9]*/) {
+        push @directives, sprintf("%s Tc", $char_spacing);
+    }
+
+    if (defined $word_spacing && $word_spacing =~ m/[0-9]+\.?[0-9]*/) {
+        push @directives, sprintf("%s Tw", $word_spacing);
+    }
+
+    $string =~ s|([()])|\\$1|g;
+
+    push @directives,
+        "$x $y Td",
+        "($string) Tj",
+        'ET';
+
     $self->{'pdf'}->page_stream($self);
     $self->{'pdf'}->uses_font($self, $font);
-    $string =~ s|([()])|\\$1|g;
-    $self->{'pdf'}->add("BT /F$font $size Tf $x $y Td ($string) Tj ET");
+    $self->{'pdf'}->add(join(' ', @directives));
 }
 
 =head2 string_underline($font, $size, $x, $y, $text, $alignment)
@@ -610,6 +662,114 @@ sub printnl {
     return $n;
 }
 
+=head2 block_text(\%params)
+
+Add block of text to the page. Parameters are explained as below:
+
+    +------------+--------------------------------------------------------------+
+    | Key        | Description                                                  |
+    +------------+--------------------------------------------------------------+
+    | page       | Object of type PDF::Create::Page                             |
+    | font       | Font index to be used.                                       |
+    | text       | Text block to be used.                                       |
+    | font_size  | Font size for the text.                                      |
+    | text_color | Text color as arrayref i.e. [r, g, b]                        |
+    | line_width | Line width (in points)                                       |
+    | start_y    | First row number (in points) when adding new page.           |
+    | end_y      | Last row number (in points) when to add new page.            |
+    | x          | x co-ordinate to start the text.                             |
+    | y          | y co-ordinate to start the text.                             |
+    +------------+--------------------------------------------------------------+
+
+=cut
+
+sub block_text {
+    my ($self, $params) = @_;
+
+    my $page       = $params->{page};
+    my $font       = $params->{font};
+    my $text       = $params->{text};
+    my $font_size  = $params->{font_size};
+    my $text_color = $params->{text_color};
+    my $line_width = $params->{line_width};
+    my $start_y    = $params->{start_y};
+    my $end_y      = $params->{end_y};
+    my $x          = $params->{x};
+    my $y          = $params->{y};
+    my $one_space  = $page->string_width($font, ' ') * $font_size;
+
+    my @lines = ();
+    foreach my $block (split /\n/, $text) {
+        my @words = split(/ /, $block);
+        my $para_last_line = 0;
+
+        while (@words) {
+            my $num_words    = 1;
+            my $string_width = 0;
+            my $space_width  = undef;
+
+            while (1) {
+                $string_width = $font_size * $page->string_width(
+                $font, _get_text(\@words, $num_words));
+
+                # Shorter, try one more word
+                if ($string_width + $one_space < $line_width) {
+                    if (scalar(@words) > $num_words) {
+                        $num_words++;
+                        next;
+                    }
+                }
+
+                last if ($num_words == 1);
+
+                # Longer, chop a word off, then space accordingly
+                $para_last_line = scalar(@words) == $num_words;
+                if ($string_width + $one_space > $line_width || $para_last_line) {
+                    unless ($para_last_line) {
+                        $num_words--;
+                    }
+
+                    $string_width = $font_size * $page->string_width(
+                        $font, _get_text(\@words, $num_words));
+
+                    $space_width = ($line_width - $string_width) / $num_words;
+                    last;
+                }
+            }
+
+            my %text_param = (
+                start => 1,
+                Tw    => $space_width,
+                F     => $font,
+                Tf    => $font_size,
+                Td    => "$x $y",
+                text  => _get_text(\@words, $num_words),
+                end   => 1,
+            );
+
+            if ($para_last_line) {
+                delete $text_param{Tw};
+            }
+
+            $page->text(%text_param);
+
+            if ($y <= $end_y) {
+                $y    = $start_y;
+                $page = $page->{'Parent'}->new_page();
+                $page->setrgbcolor(@$text_color);
+            }
+            else {
+                $y -= int($font_size * 1.5);
+                if ($para_last_line) {
+                    $y -= int($font_size * 1.5);
+                }
+            }
+
+            splice(@words, 0, $num_words);
+        }
+    }
+}
+
 =head2 image(%params)
 
 Inserts an image. Parameters can be:
@@ -889,6 +1049,18 @@ sub init_widths
     };
 }
 
+#
+#
+# PRIVATE METHODS
+
+sub _get_text ($$) {
+    my ($words, $num_words) = @_;
+
+    if (scalar @$words < $num_words) { die @_ };
+
+    return join(' ', map { $$words[$_] } (0..($num_words-1)));
+}
+
 =head1 AUTHORS
 
 Fabien Tassin
